@@ -49,6 +49,12 @@
 #define TABS_SETTINGS		3
 #define UNDO_SETTINGS		4
 #define WRAP_MODE_SETTINGS	8
+#define PRINT_SETTINGS		9
+#define LINE_NUMBERS_SETTINGS	10
+
+/*
+#define DEBUG_MDI_PREFS
+*/
 
 enum
 {
@@ -79,18 +85,23 @@ struct _GeditPreferencesDialogPrivate
 	/* Statusbar page */
 	GtkWidget	*statusbar_show_radiobutton;
 	GtkWidget	*statusbar_hide_radiobutton;
+	GtkWidget	*statusbar_cursor_position_checkbutton;
+	GtkWidget	*statusbar_overwrite_mode_checkbutton;
 
+#ifdef DEBUG_MDI_PREFS	
 	/*MDI page */
 	GtkWidget	*mdi_mode_optionmenu;
 	GtkWidget	*mdi_tab_pos_optionmenu;
 	GtkWidget	*mdi_tab_pos_label;
-
+#endif
 	/* Font & Colors page */
 	GtkWidget	*default_font_checkbutton;
 	GtkWidget	*default_colors_checkbutton;
 	GtkWidget	*fontpicker;
 	GtkWidget	*text_colorpicker;
 	GtkWidget	*background_colorpicker;
+	GtkWidget	*sel_text_colorpicker;
+	GtkWidget	*selection_colorpicker;
 	GtkWidget	*colors_table;
 	GtkWidget	*font_hbox;
 
@@ -106,6 +117,22 @@ struct _GeditPreferencesDialogPrivate
 	GtkWidget	*wrap_never_radiobutton;
 	GtkWidget	*wrap_word_radiobutton;
 	GtkWidget	*wrap_char_radiobutton;
+
+	/* Save page */
+	GtkWidget	*backup_copy_checkbutton;
+	GtkWidget	*utf8_radiobutton;
+	GtkWidget	*locale_if_possible_radiobutton;
+	GtkWidget	*locale_if_previous_radiobutton;
+
+	/* Print/page page */
+	GtkWidget	*add_header_checkbutton;
+	GtkWidget	*wrap_lines_checkbutton;
+	GtkWidget	*line_numbers_checkbutton;
+	GtkWidget	*line_numbers_spinbutton;
+
+	/* Line numbers page */
+	GtkWidget	*display_line_numbers_checkbutton;
+
 };
 
 typedef struct _CategoriesTreeItem	CategoriesTreeItem;
@@ -138,9 +165,13 @@ static gboolean gedit_preferences_dialog_setup_toolbar_page (GeditPreferencesDia
 static void gedit_preferences_dialog_toolbar_show_radiobutton_toggled (GtkToggleButton *show_button,
 							 GeditPreferencesDialog *dlg);
 static gboolean gedit_preferences_dialog_setup_statusbar_page (GeditPreferencesDialog *dlg, GladeXML *gui);
+
+#ifdef DEBUG_MDI_PREFS
 static gboolean gedit_preferences_dialog_setup_mdi_page (GeditPreferencesDialog *dlg, GladeXML *gui);
 static void gedit_preferences_dialog_mdi_mode_selection_done (GtkOptionMenu *option_menu,
 	       						GeditPreferencesDialog *dlg);
+#endif
+
 static gboolean gedit_preferences_dialog_setup_font_and_colors_page (GeditPreferencesDialog *dlg, 
 							GladeXML *gui);
 static gboolean gedit_preferences_dialog_setup_undo_page (GeditPreferencesDialog *dlg, GladeXML *gui);
@@ -149,6 +180,12 @@ static void gedit_preferences_dialog_undo_checkbutton_toggled (GtkToggleButton *
 static gboolean gedit_preferences_dialog_setup_tabs_page (GeditPreferencesDialog *dlg, GladeXML *gui);
 static gboolean gedit_preferences_dialog_setup_logo_page (GeditPreferencesDialog *dlg, GladeXML *gui);
 static gboolean gedit_preferences_dialog_setup_wrap_mode_page (GeditPreferencesDialog *dlg, GladeXML *gui);
+static gboolean gedit_preferences_dialog_setup_save_page (GeditPreferencesDialog *dlg, GladeXML *gui);
+static void gedit_preferences_dialog_line_numbers_checkbutton_toggled (GtkToggleButton *button,
+							 GeditPreferencesDialog *dlg);
+static gboolean gedit_preferences_dialog_setup_page_page (GeditPreferencesDialog *dlg, GladeXML *gui);
+static gboolean gedit_preferences_dialog_setup_line_numbers_page (GeditPreferencesDialog *dlg, 
+							          GladeXML *gui);
 
 
 static GtkDialogClass* parent_class = NULL;
@@ -157,29 +194,32 @@ static CategoriesTreeItem user_interface [] =
 {
 	{_("Toolbar"), NULL, TOOLBAR_SETTINGS},
 	{_("Status bar"), NULL, STATUS_BAR_SETTINGS},
-/*
+
+#ifdef DEBUG_MDI_PREFS	
 	{_("MDI"), NULL, MDI_SETTINGS},
-*/
+#endif
+
 	NULL
 };
 
 static CategoriesTreeItem editor_behavior [] =
 {
 	{_("Font & Colors"), NULL, FONT_COLORS_SETTINGS},
-/*	
+	
  	{_("Save"), NULL, SAVE_SETTINGS },
+/*
 	{_("Tabs"), NULL, TABS_SETTINGS},
 */	
 	{_("Undo"), NULL, UNDO_SETTINGS},
 	{_("Wrap mode"), NULL, WRAP_MODE_SETTINGS},
+	{_("Line numbers"), NULL , LINE_NUMBERS_SETTINGS},
 
 	NULL
 };
 
 static CategoriesTreeItem print [] =
 {
-	{_("Page"), NULL, LOGO},
-	{_("Paper"), NULL, LOGO},
+	{_("Page"), NULL, PRINT_SETTINGS},
 
 	NULL
 };
@@ -259,6 +299,8 @@ gedit_preferences_dialog_init (GeditPreferencesDialog *dlg)
 	gtk_window_set_modal (GTK_WINDOW (dlg), TRUE);
 
 	gtk_window_set_title (GTK_WINDOW (dlg), _("Preferences"));
+
+	gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
 }
 
 static void 
@@ -425,7 +467,8 @@ gedit_preferences_dialog_create_categories_tree (GeditPreferencesDialog *dlg)
   	column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), col_offset - 1);
   	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), FALSE);
 
-	g_signal_connect (selection, "changed", G_CALLBACK (gedit_preferences_dialog_categories_tree_selection_cb), dlg);
+	g_signal_connect (selection, "changed", 
+			G_CALLBACK (gedit_preferences_dialog_categories_tree_selection_cb), dlg);
 
 	gtk_container_add (GTK_CONTAINER (sw), treeview);
 
@@ -464,14 +507,17 @@ gedit_preferences_dialog_create_notebook (GeditPreferencesDialog *dlg)
 
 	gedit_preferences_dialog_setup_toolbar_page (dlg, gui);
 	gedit_preferences_dialog_setup_statusbar_page (dlg, gui);
-/* FIXME
+#ifdef DEBUG_MDI_PREFS
 	gedit_preferences_dialog_setup_mdi_page (dlg, gui);
-*/
+#endif
 	gedit_preferences_dialog_setup_font_and_colors_page (dlg, gui);
 	gedit_preferences_dialog_setup_undo_page (dlg, gui);
 	gedit_preferences_dialog_setup_tabs_page (dlg, gui);
 	gedit_preferences_dialog_setup_logo_page (dlg, gui);
 	gedit_preferences_dialog_setup_wrap_mode_page (dlg, gui);
+	gedit_preferences_dialog_setup_save_page (dlg, gui);
+	gedit_preferences_dialog_setup_page_page (dlg, gui);
+	gedit_preferences_dialog_setup_line_numbers_page (dlg, gui);
 
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (dlg->priv->notebook), LOGO);
 	
@@ -524,31 +570,38 @@ gedit_preferences_dialog_setup_toolbar_page (GeditPreferencesDialog *dlg, GladeX
 			G_CALLBACK (gedit_preferences_dialog_toolbar_show_radiobutton_toggled), dlg);
 
 
-	if (settings->have_toolbar)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->toolbar_show_radiobutton), TRUE);
+	if (gedit_settings->toolbar_visible)
+		gtk_toggle_button_set_active (
+			GTK_TOGGLE_BUTTON (dlg->priv->toolbar_show_radiobutton), TRUE);
 	else
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->toolbar_hide_radiobutton), TRUE);
+		gtk_toggle_button_set_active (
+			GTK_TOGGLE_BUTTON (dlg->priv->toolbar_hide_radiobutton), TRUE);
 
-	switch (settings->toolbar_labels)
+	switch (gedit_settings->toolbar_buttons_style)
 	{
 		case GEDIT_TOOLBAR_SYSTEM:
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->toolbar_system_radiobutton), TRUE);
+			gtk_toggle_button_set_active (
+				GTK_TOGGLE_BUTTON (dlg->priv->toolbar_system_radiobutton), TRUE);
 			break;
 		case GEDIT_TOOLBAR_ICONS:
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->toolbar_icon_radiobutton), TRUE);
+			gtk_toggle_button_set_active (
+				GTK_TOGGLE_BUTTON (dlg->priv->toolbar_icon_radiobutton), TRUE);
 			break;
 		case GEDIT_TOOLBAR_ICONS_AND_TEXT:
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->toolbar_icon_text_radiobutton), TRUE);
+			gtk_toggle_button_set_active (
+				GTK_TOGGLE_BUTTON (dlg->priv->toolbar_icon_text_radiobutton), TRUE);
 			break;
 
 		dafult:
 			g_return_val_if_fail (FALSE, FALSE);
 	}
 	
-	if (settings->show_tooltips)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->toolbar_tooltips_checkbutton), TRUE);
+	if (gedit_settings->toolbar_view_tooltips)
+		gtk_toggle_button_set_active (
+			GTK_TOGGLE_BUTTON (dlg->priv->toolbar_tooltips_checkbutton), TRUE);
 	else
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->toolbar_tooltips_checkbutton), FALSE);
+		gtk_toggle_button_set_active (
+			GTK_TOGGLE_BUTTON (dlg->priv->toolbar_tooltips_checkbutton), FALSE);
 		
 	return TRUE;
 }
@@ -578,20 +631,35 @@ gedit_preferences_dialog_setup_statusbar_page (GeditPreferencesDialog *dlg, Glad
 
 	dlg->priv->statusbar_show_radiobutton = glade_xml_get_widget (gui, "statusbar_show_radiobutton");
 	dlg->priv->statusbar_hide_radiobutton = glade_xml_get_widget (gui, "statusbar_hide_radiobutton");
+	dlg->priv->statusbar_cursor_position_checkbutton = glade_xml_get_widget (gui, 
+								"statusbar_cursor_position_checkbutton");
+	dlg->priv->statusbar_overwrite_mode_checkbutton = glade_xml_get_widget (gui, 
+								"statusbar_overwrite_mode_checkbutton");
 
 	g_return_val_if_fail (dlg->priv->statusbar_show_radiobutton, FALSE);
 	g_return_val_if_fail (dlg->priv->statusbar_hide_radiobutton, FALSE);
-
-	if (settings->show_status)
+	g_return_val_if_fail (dlg->priv->statusbar_cursor_position_checkbutton, FALSE);
+	g_return_val_if_fail (dlg->priv->statusbar_overwrite_mode_checkbutton, FALSE);
+	
+	if (gedit_settings->statusbar_visible)
 		gtk_toggle_button_set_active (
 				GTK_TOGGLE_BUTTON (dlg->priv->statusbar_show_radiobutton), TRUE);
 	else
 		gtk_toggle_button_set_active (
 				GTK_TOGGLE_BUTTON (dlg->priv->statusbar_hide_radiobutton), TRUE);
 
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (
+				      dlg->priv->statusbar_cursor_position_checkbutton),	
+				      gedit_settings->statusbar_view_cursor_position);
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (
+				      dlg->priv->statusbar_overwrite_mode_checkbutton),
+				      gedit_settings->statusbar_view_overwrite_mode);
 	return TRUE;
 }
 
+#ifdef DEBUG_MDI_PREFS
+
 static void
 gedit_preferences_dialog_mdi_mode_changed (GtkOptionMenu *option_menu,
 	       						GeditPreferencesDialog *dlg)
@@ -627,12 +695,14 @@ gedit_preferences_dialog_setup_mdi_page (GeditPreferencesDialog *dlg, GladeXML *
 			    GTK_SIGNAL_FUNC (gedit_preferences_dialog_mdi_mode_changed), dlg);
 
 	gtk_option_menu_set_history (GTK_OPTION_MENU (dlg->priv->mdi_mode_optionmenu),
-				     settings->mdi_mode);
+				     gedit_settings->mdi_mode);
 	gtk_option_menu_set_history (GTK_OPTION_MENU (dlg->priv->mdi_tab_pos_optionmenu),
-				     settings->tab_pos);
+				     gedit_settings->mdi_tabs_position);
 
 }
 
+#endif
+
 static void
 gedit_preferences_dialog_default_font_colors_checkbutton_toggled (GtkToggleButton *button,
 							 GeditPreferencesDialog *dlg)
@@ -662,7 +732,8 @@ static gboolean
 gedit_preferences_dialog_setup_font_and_colors_page (GeditPreferencesDialog *dlg, GladeXML *gui)
 {
 	GeditView* active_view = NULL;
-
+	GtkWidget* font_label;
+	
 	gedit_debug (DEBUG_PREFS, "");
 	
 	dlg->priv->default_font_checkbutton = glade_xml_get_widget (gui, "default_font_checkbutton");
@@ -670,14 +741,19 @@ gedit_preferences_dialog_setup_font_and_colors_page (GeditPreferencesDialog *dlg
 	
 	dlg->priv->text_colorpicker = glade_xml_get_widget (gui, "text_colorpicker");
 	dlg->priv->background_colorpicker = glade_xml_get_widget (gui, "background_colorpicker");
-	
+	dlg->priv->sel_text_colorpicker = glade_xml_get_widget (gui, "sel_text_colorpicker");
+	dlg->priv->selection_colorpicker = glade_xml_get_widget (gui, "selection_colorpicker");
+
 	dlg->priv->colors_table = glade_xml_get_widget (gui, "colors_table");
 	dlg->priv->font_hbox = glade_xml_get_widget (gui, "font_hbox");	
 
+	font_label = glade_xml_get_widget (gui, "font_label");
+
 	dlg->priv->fontpicker = gnome_font_picker_new ();
 	g_return_val_if_fail (dlg->priv->fontpicker, FALSE);
 
-	gnome_font_picker_set_mode (GNOME_FONT_PICKER (dlg->priv->fontpicker), GNOME_FONT_PICKER_MODE_FONT_INFO);
+	gnome_font_picker_set_mode (GNOME_FONT_PICKER (dlg->priv->fontpicker), 
+			GNOME_FONT_PICKER_MODE_FONT_INFO);
 	gnome_font_picker_fi_set_use_font_in_label (GNOME_FONT_PICKER (dlg->priv->fontpicker),
                                                        TRUE, 14);
 	gnome_font_picker_fi_set_show_size (GNOME_FONT_PICKER (dlg->priv->fontpicker), TRUE);
@@ -687,30 +763,55 @@ gedit_preferences_dialog_setup_font_and_colors_page (GeditPreferencesDialog *dlg
 
 	g_return_val_if_fail (dlg->priv->text_colorpicker, FALSE);
 	g_return_val_if_fail (dlg->priv->background_colorpicker, FALSE);
+	g_return_val_if_fail (dlg->priv->sel_text_colorpicker, FALSE);
+	g_return_val_if_fail (dlg->priv->selection_colorpicker, FALSE);
+
 	g_return_val_if_fail (dlg->priv->colors_table, FALSE);
 	g_return_val_if_fail (dlg->priv->font_hbox, FALSE);
 
+	g_return_val_if_fail (font_label, FALSE);
+
+	gtk_label_set_mnemonic_widget (GTK_LABEL (font_label), dlg->priv->fontpicker);
+	
 	gtk_box_pack_start (GTK_BOX (dlg->priv->font_hbox), dlg->priv->fontpicker, TRUE, TRUE, 0);
 	
 	/* setup the initial states */ 
 	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (dlg->priv->background_colorpicker),
-				    settings->bg[0], settings->bg[1], settings->bg[2], 0);
+				    gedit_settings->background_color.red,
+				    gedit_settings->background_color.green,  
+				    gedit_settings->background_color.blue, 0);
 
 	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (dlg->priv->text_colorpicker),
-				    settings->fg[0], settings->fg[1], settings->fg[2], 0);
+				    gedit_settings->text_color.red,
+				    gedit_settings->text_color.green,
+				    gedit_settings->text_color.blue, 0);
 
-	if (settings->font)
+	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (dlg->priv->selection_colorpicker),
+				    gedit_settings->selection_color.red,
+				    gedit_settings->selection_color.green,
+				    gedit_settings->selection_color.blue, 0);
+
+	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (dlg->priv->sel_text_colorpicker),
+				    gedit_settings->selected_text_color.red,
+				    gedit_settings->selected_text_color.green,
+				    gedit_settings->selected_text_color.blue, 0);
+
+	if (gedit_settings->editor_font)
 		gnome_font_picker_set_font_name (GNOME_FONT_PICKER (dlg->priv->fontpicker),
-						 settings->font);
+						 gedit_settings->editor_font);
 
 	g_signal_connect (G_OBJECT (dlg->priv->default_font_checkbutton), "toggled", 
-			G_CALLBACK (gedit_preferences_dialog_default_font_colors_checkbutton_toggled), dlg);
+			G_CALLBACK (gedit_preferences_dialog_default_font_colors_checkbutton_toggled), 
+			dlg);
 
 	g_signal_connect (G_OBJECT (dlg->priv->default_colors_checkbutton), "toggled", 
-			G_CALLBACK (gedit_preferences_dialog_default_font_colors_checkbutton_toggled), dlg);
+			G_CALLBACK (gedit_preferences_dialog_default_font_colors_checkbutton_toggled), 
+			dlg);
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->default_font_checkbutton), settings->use_default_font);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->default_colors_checkbutton), settings->use_default_colors);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON 
+			(dlg->priv->default_font_checkbutton), gedit_settings->use_default_font);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON 
+			(dlg->priv->default_colors_checkbutton), gedit_settings->use_default_colors);
 }
 
 static void
@@ -742,11 +843,12 @@ gedit_preferences_dialog_setup_undo_page (GeditPreferencesDialog *dlg, GladeXML
 	g_signal_connect (G_OBJECT (dlg->priv->undo_checkbutton), "toggled", 
 			G_CALLBACK (gedit_preferences_dialog_undo_checkbutton_toggled), dlg);
 
-	if (settings->undo_levels > 0)
+	if (gedit_settings->undo_levels > 0)
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (dlg->priv->undo_levels_spinbutton),
-					   (guint) settings->undo_levels);
+					   (guint) gedit_settings->undo_levels);
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->undo_checkbutton), settings->undo_levels > 0);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->undo_checkbutton), 
+			gedit_settings->undo_levels > 0);
 }
 
 
@@ -760,7 +862,7 @@ gedit_preferences_dialog_setup_tabs_page (GeditPreferencesDialog *dlg, GladeXML
 	g_return_val_if_fail (dlg->priv->undo_levels_spinbutton, FALSE);
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (dlg->priv->tabs_width_spinbutton),
-					   (guint) settings->tab_size);
+					   (guint) gedit_settings->tab_size);
 }
 
 static gboolean 
@@ -798,7 +900,7 @@ gedit_preferences_dialog_setup_wrap_mode_page (GeditPreferencesDialog *dlg, Glad
 	g_return_val_if_fail (dlg->priv->wrap_word_radiobutton, FALSE);
 	g_return_val_if_fail (dlg->priv->wrap_char_radiobutton, FALSE);
 
-	switch (settings->wrap_mode)
+	switch (gedit_settings->wrap_mode)
 	{
 		case GTK_WRAP_WORD:
 			gtk_toggle_button_set_active (
@@ -816,162 +918,292 @@ gedit_preferences_dialog_setup_wrap_mode_page (GeditPreferencesDialog *dlg, Glad
 	return TRUE;
 }
 
+static gboolean 
+gedit_preferences_dialog_setup_save_page (GeditPreferencesDialog *dlg, GladeXML *gui)
+{
+	gedit_debug (DEBUG_PREFS, "");
+	
+	dlg->priv->backup_copy_checkbutton = glade_xml_get_widget (gui, 
+			"backup_copy_checkbutton");
+
+	dlg->priv->utf8_radiobutton= glade_xml_get_widget (gui, 
+			"utf8_radiobutton"); 
+	dlg->priv->locale_if_possible_radiobutton= glade_xml_get_widget (gui, 
+			"locale_if_possible_radiobutton"); 
+	dlg->priv->locale_if_previous_radiobutton= glade_xml_get_widget (gui, 
+			"locale_if_previous_radiobutton");
+
+	g_return_val_if_fail (dlg->priv->backup_copy_checkbutton, FALSE);
+	
+	g_return_val_if_fail (dlg->priv->utf8_radiobutton, FALSE);
+	g_return_val_if_fail (dlg->priv->locale_if_possible_radiobutton, FALSE);
+	g_return_val_if_fail (dlg->priv->locale_if_previous_radiobutton, FALSE);
+
+	/* FIXME */
+	gtk_widget_set_sensitive (dlg->priv->locale_if_previous_radiobutton, FALSE);
+	
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->backup_copy_checkbutton),
+				      gedit_settings->create_backup_copy);
+
+	switch (gedit_settings->save_encoding)
+	{
+		case GEDIT_SAVE_ALWAYS_UTF8:
+			gtk_toggle_button_set_active (
+				GTK_TOGGLE_BUTTON (dlg->priv->utf8_radiobutton), TRUE);
+			break;
+		case GEDIT_SAVE_CURRENT_LOCALE_WHEN_POSSIBLE:
+			gtk_toggle_button_set_active (
+				GTK_TOGGLE_BUTTON (dlg->priv->locale_if_possible_radiobutton),
+				TRUE);
+			break;
+		case GEDIT_SAVE_CURRENT_LOCALE_IF_USED:
+			gtk_toggle_button_set_active (
+				GTK_TOGGLE_BUTTON (dlg->priv->locale_if_previous_radiobutton),
+				TRUE);
+			break;
+		default:
+			/* Not possible */
+			g_return_val_if_fail (FALSE, FALSE);
+	}
+	
+	return TRUE;
+}
+
+static void
+gedit_preferences_dialog_line_numbers_checkbutton_toggled (GtkToggleButton *button,
+							 GeditPreferencesDialog *dlg)
+{
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->line_numbers_checkbutton)))
+	{
+		gtk_widget_set_sensitive (dlg->priv->line_numbers_spinbutton, TRUE);
+		gtk_widget_grab_focus (dlg->priv->line_numbers_spinbutton);
+	}
+	else	
+		gtk_widget_set_sensitive (dlg->priv->line_numbers_spinbutton, FALSE);
+}
+
+static gboolean 
+gedit_preferences_dialog_setup_page_page (GeditPreferencesDialog *dlg, GladeXML *gui)
+{
+	gedit_debug (DEBUG_PREFS, "");
+	
+	dlg->priv->add_header_checkbutton = glade_xml_get_widget (gui, 
+				"add_header_checkbutton");
+	dlg->priv->wrap_lines_checkbutton= glade_xml_get_widget (gui, 
+				"wrap_lines_checkbutton");
+	dlg->priv->line_numbers_checkbutton = glade_xml_get_widget (gui, 
+				"line_numbers_checkbutton");
+	dlg->priv->line_numbers_spinbutton = glade_xml_get_widget (gui, 
+				"line_numbers_spinbutton");
+
+	g_return_val_if_fail (dlg->priv->add_header_checkbutton, FALSE);
+	g_return_val_if_fail (dlg->priv->wrap_lines_checkbutton, FALSE);
+	g_return_val_if_fail (dlg->priv->line_numbers_checkbutton, FALSE);
+	g_return_val_if_fail (dlg->priv->line_numbers_spinbutton, FALSE);
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->add_header_checkbutton),
+				gedit_settings->print_header);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->wrap_lines_checkbutton),
+				gedit_settings->print_wrap_lines);
+
+	g_signal_connect (G_OBJECT (dlg->priv->line_numbers_checkbutton), "toggled", 
+			G_CALLBACK (gedit_preferences_dialog_line_numbers_checkbutton_toggled), dlg);
+
+	if (gedit_settings->print_line_numbers > 0)
+	{
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (dlg->priv->line_numbers_spinbutton),
+					   (guint) gedit_settings->print_line_numbers);
+		gtk_widget_set_sensitive (dlg->priv->line_numbers_spinbutton, TRUE);
+	}
+	else
+	{
+		gtk_widget_set_sensitive (dlg->priv->line_numbers_spinbutton, FALSE);
+	}
+	
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->line_numbers_checkbutton),
+				gedit_settings->print_line_numbers > 0);
+}
+
+static gboolean 
+gedit_preferences_dialog_setup_line_numbers_page (GeditPreferencesDialog *dlg, GladeXML *gui)
+{
+	gedit_debug (DEBUG_PREFS, "");
+	
+	dlg->priv->display_line_numbers_checkbutton = glade_xml_get_widget (gui, 
+				"display_line_numbers_checkbutton");
+
+	g_return_val_if_fail (dlg->priv->display_line_numbers_checkbutton != NULL, FALSE);
+	
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->priv->display_line_numbers_checkbutton),
+				gedit_settings->show_line_numbers);
+}
 
 gboolean 
 gedit_preferences_dialog_update_settings (GeditPreferencesDialog *dlg)
 {
 	GeditPreferences old_prefs;
 	gint index;
+	const gchar* font;
+	guint16 dummy;
 
 	gedit_debug (DEBUG_PREFS, "");
 	
-	old_prefs = *settings;
+	old_prefs = *gedit_settings;
 	
 	/* Get data from toolbar page */
 	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->toolbar_show_radiobutton)))
-		settings->have_toolbar = FALSE;
+		gedit_settings->toolbar_visible = FALSE;
 	else
 	{
-		settings->have_toolbar = TRUE;
+		gedit_settings->toolbar_visible = TRUE;
 		
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->toolbar_system_radiobutton)))
-			settings->toolbar_labels = GEDIT_TOOLBAR_SYSTEM;
+			gedit_settings->toolbar_buttons_style = GEDIT_TOOLBAR_SYSTEM;
 		else
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->toolbar_icon_radiobutton)))
-			settings->toolbar_labels = GEDIT_TOOLBAR_ICONS;
+			gedit_settings->toolbar_buttons_style = GEDIT_TOOLBAR_ICONS;
 		else
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->toolbar_icon_text_radiobutton)))
-			settings->toolbar_labels = GEDIT_TOOLBAR_ICONS_AND_TEXT;
+			gedit_settings->toolbar_buttons_style = GEDIT_TOOLBAR_ICONS_AND_TEXT;
 		else
 			goto error;
 
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->toolbar_tooltips_checkbutton)))
-			settings->show_tooltips = TRUE;
+			gedit_settings->toolbar_view_tooltips = TRUE;
 		else
-			settings->show_tooltips = FALSE;
+			gedit_settings->toolbar_view_tooltips = FALSE;
 	}
 
 	/* Get data from status bar page */
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->statusbar_show_radiobutton)))
-		settings->show_status = TRUE;
+		gedit_settings->statusbar_visible = TRUE;
 	else
-		settings->show_status = FALSE;
+		gedit_settings->statusbar_visible = FALSE;
 
+	gedit_settings->statusbar_view_cursor_position = 
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (
+					dlg->priv->statusbar_cursor_position_checkbutton));
+
+	gedit_settings->statusbar_view_overwrite_mode = 
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (
+					dlg->priv->statusbar_overwrite_mode_checkbutton));
+		
 	/* Get data from undo page */
 	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->undo_checkbutton)))
-		settings->undo_levels = 0;
+		gedit_settings->undo_levels = 0;
 	else
 	{
 		gint undo_levels;
 
-		undo_levels = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (dlg->priv->undo_levels_spinbutton));
+		undo_levels = gtk_spin_button_get_value_as_int (
+				GTK_SPIN_BUTTON (dlg->priv->undo_levels_spinbutton));
 	
 		if (undo_levels < 1)
 			undo_levels = 1;
 
-		settings->undo_levels = undo_levels;
+		gedit_settings->undo_levels = undo_levels;
 	}
-#if 0   /* BONOBO_MDI has to be fixed before MDI mode can be used */
+	
+#ifdef DEBUG_MDI_PREFS
+     	/* BONOBO_MDI has to be fixed before MDI mode can be used */
 	/* Get Data from MDI page */
 	index = gtk_option_menu_get_history (GTK_OPTION_MENU (dlg->priv->mdi_mode_optionmenu));
 	if (index > 2) index = BONOBO_MDI_NOTEBOOK;
-	settings->mdi_mode = index;
+	gedit_settings->mdi_mode = index;
 	
 	if (index == BONOBO_MDI_NOTEBOOK)
 	{
 		index = gtk_option_menu_get_history (GTK_OPTION_MENU (dlg->priv->mdi_tab_pos_optionmenu));
 		if (index > 3) index = 2; /* 2 = Top */
-		settings->tab_pos = index;
+		gedit_settings->mdi_tabs_position = index;
 	}
 #endif		
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->default_colors_checkbutton)))
-	{
-		/* FIXME: is it rigth? */
-		
-		GtkStyle *style;
-		GdkColor *bg;
-		GdkColor *fg;
-
-		settings->use_default_colors = TRUE;
-		
-		style = gtk_style_new ();
-
-		bg = &style->base[0];
-		settings->bg[0] = bg->red;
-		settings->bg[1] = bg->green;
-		settings->bg[2] = bg->blue;
-
-		fg = &style->text[0];
-		settings->fg[0] = fg->red;
-		settings->fg[1] = fg->green;
-		settings->fg[2] = fg->blue;
-
-		gtk_style_unref (style);
-	}
+		gedit_settings->use_default_colors = TRUE;
 	else
-	{
-		guint16 a;
+		gedit_settings->use_default_colors = FALSE;
 
-		settings->use_default_colors = FALSE;
+	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (dlg->priv->text_colorpicker), 
+				&gedit_settings->text_color.red,
+				&gedit_settings->text_color.green,
+		       		&gedit_settings->text_color.blue, 	       
+				&dummy);
 
-		gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (dlg->priv->text_colorpicker), 
-				&settings->fg[0], &settings->fg[1], &settings->fg[2], &a);
+	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (dlg->priv->background_colorpicker),
+		       		&gedit_settings->background_color.red,
+				&gedit_settings->background_color.green,
+		       		&gedit_settings->background_color.blue, 	       
+				&dummy);
+
+	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (dlg->priv->sel_text_colorpicker), 
+				&gedit_settings->selected_text_color.red,
+				&gedit_settings->selected_text_color.green,
+		       		&gedit_settings->selected_text_color.blue, 	       
+				&dummy);
+
+	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (dlg->priv->selection_colorpicker),
+		       		&gedit_settings->selection_color.red,
+				&gedit_settings->selection_color.green,
+		       		&gedit_settings->selection_color.blue, 	       
+				&dummy);
 
-		gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (dlg->priv->background_colorpicker), 
-				&settings->bg[0], &settings->bg[1], &settings->bg[2], &a);
-	}
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->default_font_checkbutton)))
-	{
-		/* FIXME: is it rigth? */
-
-		GtkStyle *style;
-		gchar *font;
-
-		settings->use_default_font = TRUE;
-		
-		style = gtk_style_new ();
-
-		font = pango_font_description_to_string (style->font_desc);
-		
-		if (font != NULL)
-		{
-			g_free (settings->font);
-			settings->font = g_strdup (font);
-			g_free (font);
-		}
-
-		gtk_style_unref (style);
-
-	}
+		gedit_settings->use_default_font = TRUE;
 	else
-	{
-		const gchar* font;
-
-		settings->use_default_font = FALSE;
+		gedit_settings->use_default_font = FALSE;
 		
-		font = gnome_font_picker_get_font_name (GNOME_FONT_PICKER (dlg->priv->fontpicker));		
-	
-		if (font != NULL)
-		{
-			g_free (settings->font);
-			settings->font = g_strdup (font);
-		}
+	font = gnome_font_picker_get_font_name (GNOME_FONT_PICKER (dlg->priv->fontpicker));		
+	if (font != NULL)
+	{
+		g_free (gedit_settings->editor_font);
+		gedit_settings->editor_font = g_strdup (font);
 	}
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->wrap_word_radiobutton)))
-		settings->wrap_mode = GTK_WRAP_WORD;
+		gedit_settings->wrap_mode = GTK_WRAP_WORD;
 	else
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->wrap_char_radiobutton)))
-		settings->wrap_mode = GTK_WRAP_CHAR;
+		gedit_settings->wrap_mode = GTK_WRAP_CHAR;
 	else
-		settings->wrap_mode = GTK_WRAP_NONE;
+		gedit_settings->wrap_mode = GTK_WRAP_NONE;
 			
-	return TRUE;
+	/* Save page */
+	gedit_settings->create_backup_copy = 
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->backup_copy_checkbutton));
 
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->utf8_radiobutton)))
+		gedit_settings->save_encoding = GEDIT_SAVE_ALWAYS_UTF8;
+	else
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->locale_if_possible_radiobutton)))
+		gedit_settings->save_encoding = GEDIT_SAVE_CURRENT_LOCALE_WHEN_POSSIBLE;
+	else
+		gedit_settings->save_encoding = GEDIT_SAVE_CURRENT_LOCALE_IF_USED;
+	
+	/* Print page / page */
+	gedit_settings->print_header = 
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->add_header_checkbutton));	
+
+	gedit_settings->print_wrap_lines = 
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->wrap_lines_checkbutton));	
+
+	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->line_numbers_checkbutton)))
+		gedit_settings->print_line_numbers = 0;
+	else
+	{
+		gedit_settings->print_line_numbers = MAX (1, gtk_spin_button_get_value_as_int (
+				GTK_SPIN_BUTTON (dlg->priv->line_numbers_spinbutton)));
+	}
+
+	gedit_settings->show_line_numbers = 
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (
+					      dlg->priv->display_line_numbers_checkbutton));	
+
+	return TRUE;
+	
 error:
-	*settings = old_prefs;
+	*gedit_settings = old_prefs;
 	g_return_val_if_fail (FALSE, FALSE);
 }
 
