@@ -40,7 +40,13 @@
 #include "gedit-mdi.h"
 #include "gedit-recent.h" 
 #include "gedit-prefs.h"
+#include "gedit-file-selector-util.h"
 
+#define USE_BONOBO_FILE_SEL_API	
+
+#ifndef USE_BONOBO_FILE_SEL_API
+static GtkWidget *open_file_selector = NULL;
+static GtkWidget *save_file_selector = NULL;
 
 static gint gedit_file_selector_delete_event_handler (GtkWidget *widget, 
 						      GdkEventAny *event);
@@ -52,13 +58,13 @@ static void gedit_file_open_ok_button_clicked_handler (GtkWidget *widget,
 						   GeditMDIChild *active_child);
 static void gedit_file_save_as_ok_button_clicked_handler (GtkWidget *widget, 
 							GeditMDIChild *child);
-
 static GSList*  gedit_file_selector_get_filenames (GtkFileSelection *file_selector);
 static gboolean gedit_file_open_is_dir (GtkFileSelection *file_selector);
+#endif
+
 static gboolean gedit_file_open_real (const gchar* file_name, GeditMDIChild* child);
+static gboolean gedit_file_save_as_real (const gchar* file_name, GeditMDIChild *child);
 
-static GtkWidget *open_file_selector = NULL;
-static GtkWidget *save_file_selector = NULL;
 
 void 
 gedit_file_new (void)
@@ -113,6 +119,8 @@ gedit_file_close (GtkWidget *view)
 		gedit_mdi_set_active_window_verbs_sensitivity (BONOBO_MDI (gedit_mdi));
 }
 
+#ifndef USE_BONOBO_FILE_SEL_API
+
 static gint
 gedit_file_selector_delete_event_handler (GtkWidget *widget, GdkEventAny *event)
 {
@@ -253,22 +261,16 @@ gedit_file_open_ok_button_clicked_handler (GtkWidget *widget, GeditMDIChild *act
 	gtk_widget_hide (GTK_WIDGET (file_selector));
 
 	gtk_main_quit ();
-	
-	/* FIXME
-	if (gedit_document_current())
-	{
-		g_return_if_fail(gedit_window_active_app());
-		gedit_window_set_widgets_sensitivity_ro (gedit_window_active_app(), 
-				gedit_document_current()->readonly);
-	}	
-	*/
-
+		
 	return;
 }
+#endif
 
 void
 gedit_file_open (GeditMDIChild *active_child)
 {
+#ifndef USE_BONOBO_FILE_SEL_API
+
 	gint ok_signal_id = 0;
 	
 	gedit_debug (DEBUG_FILE, "");
@@ -335,6 +337,32 @@ gedit_file_open (GeditMDIChild *active_child)
 			ok_signal_id);
 
 	return;
+#else
+	gchar** files;
+	
+	gedit_debug (DEBUG_FILE, "");
+
+	files = gedit_file_selector_open_multi (
+			GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
+			TRUE,
+		        _("Open File ..."), 
+			NULL, 
+			NULL);
+	
+	if (files) 
+	{
+		gint i;
+		for (i = 0; files[i]; ++i)
+		{
+			if (gedit_file_open_real (files[i], active_child))
+				gedit_utils_flash_va (_("Loaded file %s"), files[i]);
+			
+			gedit_debug (DEBUG_FILE, "File: %s", files[i]);
+		}
+
+		g_strfreev (files);
+	}
+#endif	
 }
 
 static gboolean
@@ -365,8 +393,8 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 			gchar *formatted_uri = gnome_vfs_x_format_uri_for_display (uri);
 		       	
 			errstr = g_strdup_printf (_("An error occurred while "
-							   "opening the file \"%s\".\n\n%s."),
-							   formatted_uri, error->message);
+						    "opening the file \"%s\".\n\n%s."),
+						  formatted_uri, error->message);
 		
 			dialog = gtk_message_dialog_new (
 					GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
@@ -375,6 +403,10 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 				   	GTK_BUTTONS_OK,
 					errstr);
 			
+			gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+			gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
 			gtk_dialog_run (GTK_DIALOG (dialog));
   			gtk_widget_destroy (dialog);
 			
@@ -409,8 +441,8 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 			gchar *formatted_uri = gnome_vfs_x_format_uri_for_display (uri);
 		       	
 			errstr = g_strdup_printf (_("An error occurred while "
-							   "opening the file \"%s\".\n\n%s."),
-							   formatted_uri, error->message);
+						    "opening the file \"%s\".\n\n%s."),
+						  formatted_uri, error->message);
 		
 			dialog = gtk_message_dialog_new (
 					GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
@@ -419,6 +451,10 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 				   	GTK_BUTTONS_OK,
 					errstr);
 			
+			gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+			gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
 			gtk_dialog_run (GTK_DIALOG (dialog));
   			gtk_widget_destroy (dialog);
 
@@ -431,8 +467,6 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 		}	
 	}
 
-	/* FIXME */
-
 	gedit_recent_add (uri);
 	gedit_recent_update_all_windows (BONOBO_MDI (gedit_mdi)); 
 	
@@ -460,7 +494,6 @@ gedit_file_save (GeditMDIChild* child)
 	
 	if (gedit_document_is_untitled (doc))
 	{
-		/* TODO */		
 		gedit_debug (DEBUG_FILE, "Untitled");
 
 		return gedit_file_save_as (child);
@@ -491,7 +524,7 @@ gedit_file_save (GeditMDIChild* child)
 	       	
 		if (error == NULL)
 		{
-			error_message = _("Unknow error");
+			error_message = _("Unknown error");
 		}
 		else
 		{
@@ -514,6 +547,10 @@ gedit_file_save (GeditMDIChild* child)
 				GTK_BUTTONS_OK,
 				errstr);
 			
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
 		gtk_dialog_run (GTK_DIALOG (dialog));
   		gtk_widget_destroy (dialog);
 
@@ -546,6 +583,8 @@ gedit_file_save (GeditMDIChild* child)
 gboolean
 gedit_file_save_as (GeditMDIChild *child)
 {
+#ifndef USE_BONOBO_FILE_SEL_API
+
 	gint ok_signal_id = 0;
 	GeditDocument* doc = NULL;
 	gchar *fname = NULL;
@@ -613,13 +652,53 @@ gedit_file_save_as (GeditMDIChild *child)
 			ok_signal_id);
 
 	return !gedit_document_get_modified (child->document);
+
+#else		
+	gchar* file;
+	gboolean ret = FALSE;
+	GeditDocument *doc;
+	gchar* fname;
+
+	gedit_debug (DEBUG_FILE, "");
+
+	g_return_val_if_fail (child != NULL, FALSE);
+
+	doc = child->document;
+	g_return_val_if_fail (doc != NULL, FALSE);
+
+	fname = gedit_document_get_uri (doc);
+
+	file = gedit_file_selector_save (
+			GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
+			FALSE,
+		        _("Save as ..."), 
+			NULL, 
+			NULL,
+			fname);
+	g_free (fname);
+
+	if (file) 
+	{
+		gedit_utils_flash_va (_("Saving file '%s' ..."), file);
+		
+		ret = gedit_file_save_as_real (file, child);
+		
+		if (ret)
+			gedit_utils_flash_va (_("File '%s' saved."), file);
+		else
+			gedit_utils_flash_va (_("The document has not been saved."));
+
+		gedit_debug (DEBUG_FILE, "File: %s", file);
+		g_free (file);
+	}
+
+	return ret;
+#endif
 }
 
-static void
-gedit_file_save_as_ok_button_clicked_handler (GtkWidget *widget, GeditMDIChild *child)
+static gboolean
+gedit_file_save_as_real (const gchar* file_name, GeditMDIChild *child)
 {
-	GtkFileSelection *file_selector;
-	const gchar *file_name;
 	gchar *uri;
 	gboolean ret;
 	GeditDocument *doc = NULL;
@@ -627,10 +706,86 @@ gedit_file_save_as_ok_button_clicked_handler (GtkWidget *widget, GeditMDIChild *
 
 	gedit_debug (DEBUG_FILE, "");
 
-	g_return_if_fail (child != NULL);
+	g_return_val_if_fail (child != NULL, FALSE);
 
 	doc = child->document;
-	g_return_if_fail (doc != NULL);
+	g_return_val_if_fail (doc != NULL, FALSE);
+
+	uri = gnome_vfs_x_make_uri_canonical (file_name);
+	g_return_val_if_fail (uri != NULL, FALSE);
+	
+	ret = gedit_document_save_as (doc, uri, &error);
+
+	if (!ret)
+	{
+		GtkWidget *dialog;
+		gchar *error_message;
+		gchar *errstr;
+
+		gedit_debug (DEBUG_FILE, "FAILED");
+	       	
+		if (error == NULL)
+		{
+			error_message = _("Unknown error");
+		}
+		else
+		{
+			error_message = error->message;
+		}
+		
+		errstr = g_strdup_printf (_("An error occurred while "
+					    "saving the file \"%s\".\n\n%s."),
+					    file_name, error_message);
+		
+		dialog = gtk_message_dialog_new (
+					GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
+					GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				   	GTK_MESSAGE_ERROR,
+				   	GTK_BUTTONS_OK,
+					errstr);
+
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+		gtk_dialog_run (GTK_DIALOG (dialog));
+  		gtk_widget_destroy (dialog);
+
+		g_free (errstr);
+
+		if (error != NULL)
+			g_error_free (error);
+		
+		g_free (uri);
+
+		return FALSE;
+		
+	}	
+	else
+	{
+		gedit_debug (DEBUG_FILE, "OK");
+
+		gedit_recent_add (uri);
+		gedit_recent_update_all_windows (BONOBO_MDI (gedit_mdi)); 
+
+		g_free (uri);
+
+		return TRUE;
+	}
+}
+
+#ifndef USE_BONOBO_FILE_SEL_API
+
+static void
+gedit_file_save_as_ok_button_clicked_handler (GtkWidget *widget, GeditMDIChild *child)
+{
+	GtkFileSelection *file_selector;
+	const gchar *file_name;
+	gboolean ret;
+
+	gedit_debug (DEBUG_FILE, "");
+
+	g_return_if_fail (child != NULL);
 
 	g_return_if_fail (GTK_IS_FILE_SELECTION (save_file_selector));
 	file_selector = GTK_FILE_SELECTION (save_file_selector);
@@ -660,6 +815,8 @@ gedit_file_save_as_ok_button_clicked_handler (GtkWidget *widget, GeditMDIChild *
 
 		gtk_dialog_set_default_response	(GTK_DIALOG (msgbox), GTK_RESPONSE_CANCEL);
 
+		gtk_window_set_resizable (GTK_WINDOW (msgbox), FALSE);
+
 		ret = gtk_dialog_run (GTK_DIALOG (msgbox));
 		
 		gtk_widget_destroy (msgbox);
@@ -670,93 +827,60 @@ gedit_file_save_as_ok_button_clicked_handler (GtkWidget *widget, GeditMDIChild *
 
 	gtk_widget_hide (GTK_WIDGET (file_selector));
 
-	uri = gnome_vfs_x_make_uri_canonical (file_name);
-	g_return_if_fail (uri != NULL);
-	
-	gedit_utils_flash_va (_("Saving file '%s' ..."), file_name);	
-	
-	ret = gedit_document_save_as (doc, uri, &error);
-
-	if (!ret)
-	{
-		GtkWidget *dialog;
-		gchar *error_message;
-		gchar *errstr;
-
-		gedit_debug (DEBUG_FILE, "FAILED");
-	       	
-		if (error == NULL)
-		{
-			error_message = _("Unknow error");
-		}
-		else
-		{
-			error_message = error->message;
-		}
+	gedit_utils_flash_va (_("Saving file '%s' ..."), file_name);
 		
-		errstr = g_strdup_printf (_("An error occurred while "
-					    "saving the file \"%s\".\n\n%s."),
-					    file_name, error_message);
+	ret = gedit_file_save_as_real (file_name, child);
 		
-		dialog = gtk_message_dialog_new (
-					GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
-					GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-				   	GTK_MESSAGE_ERROR,
-				   	GTK_BUTTONS_OK,
-					errstr);
-			
-		gtk_dialog_run (GTK_DIALOG (dialog));
-  		gtk_widget_destroy (dialog);
-
-		g_free (errstr);
-
-		if (error != NULL)
-			g_error_free (error);
-
-		gedit_utils_flash_va (_("The document has not been saved."));
-		
-	}	
-	else
-	{
-		gedit_debug (DEBUG_FILE, "OK");
-
+	if (ret)
 		gedit_utils_flash_va (_("File '%s' saved."), file_name);
+	else
+		gedit_utils_flash_va (_("The document has not been saved."));
 
-		gedit_recent_add (uri);
-		gedit_recent_update_all_windows (BONOBO_MDI (gedit_mdi)); 
-	}
-
-	g_free (uri);
-	
 	gtk_main_quit ();
 	
 	return;
 }
 
+#endif
+
 gboolean
 gedit_file_close_all (void)
 {
+	gboolean ret;
 	gedit_debug (DEBUG_FILE, "");
 
-	return bonobo_mdi_remove_all (BONOBO_MDI (gedit_mdi), FALSE);
+	ret = bonobo_mdi_remove_all (BONOBO_MDI (gedit_mdi), FALSE);
+
+	if (bonobo_mdi_get_active_child (BONOBO_MDI (gedit_mdi)) == NULL)
+		gedit_mdi_set_active_window_verbs_sensitivity (BONOBO_MDI (gedit_mdi));
+
+	return ret;
 }
 
 void
 gedit_file_exit (void)
 {
 	gedit_debug (DEBUG_FILE, "");
-
+	
 	if (!gedit_file_close_all ())
 		return;
+
+	gedit_debug (DEBUG_FILE, "All files closed.");
+	
 	/* We need to disconnect the signal because mdi "destroy" event
 	   is connected to gedit_file_exit ( i.e. this function ). */
 	gtk_signal_disconnect_by_func (GTK_OBJECT (gedit_mdi),
 			    GTK_SIGNAL_FUNC (gedit_file_exit), NULL);
-
-	gtk_object_destroy (GTK_OBJECT (gedit_mdi));
+		
 	gedit_prefs_save_settings ();
 
-	gedit_recent_history_write_config ();
+	gedit_recent_history_save ();
+
+	gedit_debug (DEBUG_FILE, "Unref gedit_mdi.");
+
+	g_object_unref (G_OBJECT (gedit_mdi));
+
+	gedit_debug (DEBUG_FILE, "Unref gedit_mdi: DONE");
 
 	gtk_main_quit ();
 }
@@ -819,7 +943,7 @@ gedit_file_revert (GeditMDIChild *child)
 	       	
 		if (error == NULL)
 		{
-			error_message = _("Unknow error");
+			error_message = _("Unknown error");
 		}
 		else
 		{
@@ -841,7 +965,11 @@ gedit_file_revert (GeditMDIChild *child)
 				   	GTK_MESSAGE_ERROR,
 				   	GTK_BUTTONS_OK,
 					errstr);
-			
+		
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
 		gtk_dialog_run (GTK_DIALOG (dialog));
   		gtk_widget_destroy (dialog);
 
@@ -890,17 +1018,23 @@ gedit_file_open_uri_list (GList* uri_list)
 {
 	gchar *full_path;
 	gboolean ret = TRUE;
+	
+	BonoboMDIChild *active_child = NULL;
+	
 	gedit_debug (DEBUG_FILE, "");
 	
 	if (uri_list == NULL) 
 		return FALSE;
-	
+
+	active_child = bonobo_mdi_get_active_child (BONOBO_MDI (gedit_mdi));
+
         /* create a file for each document in the parameter list */
 	for (;uri_list; uri_list = uri_list->next)
 	{
 		full_path = gedit_file_make_canonical_uri_from_shell_arg (uri_list->data);
 		if (full_path != NULL) {
-			ret |= gedit_file_open_real (full_path, NULL);
+			ret |= gedit_file_open_real (full_path, 
+					(active_child != NULL) ? GEDIT_MDI_CHILD (active_child): NULL);
 			g_free (full_path);
 		}
 	}
@@ -960,6 +1094,7 @@ gedit_file_open_single_uri (const gchar* uri)
 	return ret;
 }
 
+/* FIXME: it is broken */
 gboolean
 gedit_file_open_from_stdin (GeditMDIChild *active_child)
 {
@@ -1015,7 +1150,11 @@ gedit_file_open_from_stdin (GeditMDIChild *active_child)
 			   	GTK_MESSAGE_ERROR,
 			   	GTK_BUTTONS_OK,
 				errstr);
-			
+
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 
