@@ -32,8 +32,6 @@
 #define __GEDIT_FILE_H__
 
 #include "gedit-mdi-child.h"
-#include "recent-files/egg-recent-view.h"
-#include "recent-files/egg-recent-item.h"
 
 void 		gedit_file_new 		(void);
 void 		gedit_file_open 	(GeditMDIChild *active_child);
@@ -45,8 +43,8 @@ gboolean	gedit_file_close_all 	(void);
 void		gedit_file_exit 	(void);
 gboolean	gedit_file_revert 	(GeditMDIChild *child);
 
-gboolean 	gedit_file_open_uri_list (GList* uri_list, gint line, gboolean create);
-gboolean 	gedit_file_open_recent   (EggRecentView *view, EggRecentItem *item, gpointer data);
+gboolean 	gedit_file_open_uri_list (GList* uri_list);
+gboolean 	gedit_file_open_recent   (GeditMDIChild *child, gchar* uri);
 gboolean 	gedit_file_open_single_uri (const gchar* uri);
 
 gboolean	gedit_file_open_from_stdin (GeditMDIChild *active_child);
