@@ -45,9 +45,12 @@
 #include "gedit-recent.h" 
 #include "gedit-file.h"
 #include "gedit-view.h"
+#include "gedit-utils.h"
 
 #include <bonobo/bonobo-ui-util.h>
+#include <bonobo/bonobo-control.h>
 
+#include <gconf/gconf-client.h>
 
 struct _GeditMDIPrivate
 {
@@ -124,21 +127,18 @@ gedit_mdi_init (GeditMDI  *mdi)
 {
 	gedit_debug (DEBUG_MDI, "START");
 
-	bonobo_mdi_construct (BONOBO_MDI (mdi), "gedit2", "gedit", settings->tab_pos);
+	bonobo_mdi_construct (BONOBO_MDI (mdi), "gedit2", "gedit", 
+			gedit_settings->mdi_tabs_position);
 	
 	mdi->priv = g_new0 (GeditMDIPrivate, 1);
 
 	mdi->priv->untitled_number = 0;	
 
-	/* FIXME */
-	gedit_debug (DEBUG_MDI, "------ %s\n", 
-			bonobo_ui_util_get_ui_fname ("", GEDIT_UI_DIR "gedit-ui.xml"));
-	
 	bonobo_mdi_set_ui_template_file (BONOBO_MDI (mdi), GEDIT_UI_DIR "gedit-ui.xml", gedit_verbs);
 	
 	bonobo_mdi_set_child_list_path (BONOBO_MDI (mdi), "/menu/Documents/");
 
-	bonobo_mdi_set_mode (BONOBO_MDI (mdi), settings->mdi_mode);
+	bonobo_mdi_set_mode (BONOBO_MDI (mdi), gedit_settings->mdi_mode);
 
 	/* Connect signals */
 	gtk_signal_connect (GTK_OBJECT (mdi), "top_window_created",
@@ -206,6 +206,9 @@ gedit_mdi_new (void)
 static void
 gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 {
+	GtkWidget *widget;
+	BonoboControl *control;
+	
 	static GtkTargetEntry drag_types[] =
 	{
 		{ "text/uri-list", 0, 0 },
@@ -226,24 +229,66 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 	gtk_signal_connect (GTK_OBJECT (win), "drag_data_received",
 			    GTK_SIGNAL_FUNC (gedit_mdi_drag_data_received_handler), 
 			    NULL);
+	
+	/* Add cursor position status bar */
+	widget = gtk_statusbar_new ();
+	control = bonobo_control_new (widget);
+	
+	gtk_widget_set_size_request (widget, 150, 10);	
+	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (widget), FALSE);
+	
+	bonobo_ui_component_object_set (bonobo_mdi_get_ui_component_from_window (win),
+		       			"/status/CursorPosition",
+					BONOBO_OBJREF (control),
+					NULL);
 
+	bonobo_object_unref (BONOBO_OBJECT (control));
+
+	g_object_set_data (G_OBJECT (win), "CursorPosition", widget);
+
+	/* Add overwrite mode status bar */
+	widget = gtk_statusbar_new ();
+	control = bonobo_control_new (widget);
+	
+	gtk_widget_set_size_request (widget, 80, 10);
+	
+	bonobo_ui_component_object_set (bonobo_mdi_get_ui_component_from_window (win),
+		       			"/status/OverwriteMode",
+					BONOBO_OBJREF (control),
+					NULL);
+
+	bonobo_object_unref (BONOBO_OBJECT (control));
+
+	g_object_set_data (G_OBJECT (win), "OverwriteMode", widget);
+
+	/* Set the statusbar style according to prefs */
 	gedit_mdi_set_app_statusbar_style (win);
 	
 	/* Set the toolbar style according to prefs */
 	gedit_mdi_set_app_toolbar_style (win);
 		
 	/* Set the window prefs. */
-	gtk_window_set_default_size (GTK_WINDOW (win), settings->width, settings->height);
+	gtk_window_set_default_size (GTK_WINDOW (win), 
+			gedit_settings->window_width, 
+			gedit_settings->window_height);
 	gtk_window_set_policy (GTK_WINDOW (win), TRUE, TRUE, FALSE);
 
+	g_signal_connect (GTK_WIDGET (win), "configure_event",
+	                  GTK_SIGNAL_FUNC (gedit_prefs_configure_event_handler), 
+			  NULL);
 	
 	/* Add the recent files */
 	gedit_recent_init (win);
-#if 0 /* FIXME */
+
+	
+#if 0 
+	/* FIXME */
 
 	/* Add the plugins to the menus */
 	gedit_plugins_menu_add (app);
+	
 #endif
+
 }
 
 static void 
@@ -287,7 +332,9 @@ gedit_mdi_set_app_toolbar_style (BonoboWindow *win)
 {
 	BonoboUIEngine *ui_engine;
 	BonoboUIError ret;
-	
+	GConfClient *client;
+	gboolean labels;
+
 	gedit_debug (DEBUG_MDI, "");
 	
 	g_return_if_fail (BONOBO_IS_WINDOW (win));
@@ -295,7 +342,7 @@ gedit_mdi_set_app_toolbar_style (BonoboWindow *win)
 	ui_engine = bonobo_window_get_ui_engine (win);
 	g_return_if_fail (ui_engine != NULL);
 	
-	if (!settings->have_toolbar)
+	if (!gedit_settings->toolbar_visible)
 	{
 		ret = bonobo_ui_engine_xml_set_prop (ui_engine, "/Toolbar",
 				"hidden", "1");
@@ -311,37 +358,54 @@ gedit_mdi_set_app_toolbar_style (BonoboWindow *win)
 	if (ret != BONOBO_UI_ERROR_OK) 
 		goto error;
 
-	switch (settings->toolbar_labels)
+	ret = bonobo_ui_engine_xml_set_prop (ui_engine, "/Toolbar",
+					"tips", gedit_settings->toolbar_view_tooltips ? "1" : "0");
+
+	if (ret != BONOBO_UI_ERROR_OK) 
+		goto error;
+
+
+	switch (gedit_settings->toolbar_buttons_style)
 	{
 		case GEDIT_TOOLBAR_SYSTEM:
-			/* FIXME
-			if (gnome_preferences_get_toolbar_labels())
-			{
-			*/
+						
+			client = gconf_client_get_default ();
+			if (client == NULL) 
+				goto error;
+
+			labels = gconf_client_get_bool (client, 
+					"/desktop/gnome/interface/toolbar-labels", NULL);
+
+			g_object_unref (G_OBJECT (client));
+			
+			if (labels)
+			{			
 				gedit_debug (DEBUG_MDI, "SYSTEM: BOTH");
 				ret = bonobo_ui_engine_xml_set_prop (ui_engine, "/Toolbar",
 						"look", "both");
 				if (ret != BONOBO_UI_ERROR_OK) 
 					goto error;
-			/*
+			
 			}
 			else
 			{
 				gedit_debug (DEBUG_MDI, "SYSTEM: ICONS");
 				ret = bonobo_ui_engine_xml_set_prop (ui_engine, "/Toolbar",
 						"look", "icons");
-				g_return_if_fail (ret == BONOBO_UI_ERROR_OK);
-			}*/
+				if (ret != BONOBO_UI_ERROR_OK) 
+					goto error;
+			}
 			break;
 		case GEDIT_TOOLBAR_ICONS:
 			gedit_debug (DEBUG_MDI, "GEDIT: ICONS");
 			ret = bonobo_ui_engine_xml_set_prop (ui_engine, "/Toolbar",
-						"look", "icons");
+						"look", "icon");
 			if (ret != BONOBO_UI_ERROR_OK) 
 					goto error;
 
 			break;
 		case GEDIT_TOOLBAR_ICONS_AND_TEXT:
+			gedit_debug (DEBUG_MDI, "GEDIT: ICONS_AND_TEXT");
 			ret = bonobo_ui_engine_xml_set_prop (ui_engine, "/Toolbar",
 						"look", "both");
 			if (ret != BONOBO_UI_ERROR_OK) 
@@ -351,15 +415,9 @@ gedit_mdi_set_app_toolbar_style (BonoboWindow *win)
 			goto error;
 		break;
 	}
-
-	ret = bonobo_ui_engine_xml_set_prop (ui_engine, "/Toolbar",
-					"tips", settings->show_tooltips ? "1" : "0");
-
-	if (ret != BONOBO_UI_ERROR_OK) 
-		goto error;
-
+	
 	bonobo_ui_engine_thaw (ui_engine);
-
+	
 	return;
 	
 error:
@@ -372,6 +430,7 @@ gedit_mdi_set_app_statusbar_style (BonoboWindow *win)
 {
 	BonoboUIEngine *ui_engine;
 	BonoboUIError ret;
+	GtkWidget *cp, *om;
 	
 	gedit_debug (DEBUG_MDI, "");
 	
@@ -381,8 +440,43 @@ gedit_mdi_set_app_statusbar_style (BonoboWindow *win)
 	g_return_if_fail (ui_engine != NULL);
 
 	ret = bonobo_ui_engine_xml_set_prop (ui_engine, "/status",
-				"hidden", settings->show_status ? "0" : "1");
+				"hidden", gedit_settings->statusbar_visible ? "0" : "1");
 	g_return_if_fail (ret == BONOBO_UI_ERROR_OK);		
+
+	cp = GTK_WIDGET (g_object_get_data (G_OBJECT (win), "CursorPosition"));
+	g_return_if_fail (cp != NULL);
+
+	if (gedit_settings->statusbar_view_cursor_position)
+		gtk_widget_show (cp);
+	else
+		gtk_widget_hide (cp);
+
+	om = GTK_WIDGET (g_object_get_data (G_OBJECT (win), "OverwriteMode"));
+	g_return_if_fail (om != NULL);
+
+	if (gedit_settings->statusbar_view_overwrite_mode)
+		gtk_widget_show (om);
+	else
+		gtk_widget_hide (om);
+
+	if (!gedit_settings->statusbar_view_overwrite_mode)
+		gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (cp), TRUE);
+	else
+		gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (cp), FALSE);
+
+	if (!gedit_settings->statusbar_view_cursor_position &&
+	    !gedit_settings->statusbar_view_overwrite_mode)
+	{
+		ret = bonobo_ui_engine_xml_set_prop (ui_engine, "/status/item",
+				"resize_grip", "1");
+		g_return_if_fail (ret == BONOBO_UI_ERROR_OK);		
+	}
+	else
+	{
+		ret = bonobo_ui_engine_xml_set_prop (ui_engine, "/status/item",
+				"resize_grip", "0");
+		g_return_if_fail (ret == BONOBO_UI_ERROR_OK);	
+	}	
 }
 
 static void 
@@ -469,6 +563,7 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 		GtkWidget *msgbox, *w;
 		gchar *fname = NULL, *msg = NULL;
 		gint ret;
+		gboolean exiting;
 
 		w = GTK_WIDGET (g_list_nth_data (bonobo_mdi_child_get_views (child), 0));
 			
@@ -485,20 +580,40 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 				  "Your changes will be lost if you don't save them."),
 				fname);
 
-	      	gtk_dialog_add_button (GTK_DIALOG (msgbox),
-                             _("_Don't save"),
-                             GTK_RESPONSE_NO);
+		gedit_dialog_add_button (GTK_DIALOG (msgbox),
+				_("Do_n't save"), GTK_STOCK_NO,
+				GTK_RESPONSE_NO);
+
+		if (gedit_close_x_button_pressed)
+			exiting = FALSE;
+		else if (gedit_exit_button_pressed)
+			exiting = TRUE;
+		else
+		{
+			/* Delete event generated */
+			if (g_list_length (bonobo_mdi_get_windows (BONOBO_MDI (gedit_mdi))) == 1)
+				exiting = TRUE;
+			else
+				exiting = FALSE;
+		}
+			
+		if (exiting)
+			gedit_dialog_add_button (GTK_DIALOG (msgbox),
+					_("_Don't quit"), GTK_STOCK_CANCEL,
+                	             	GTK_RESPONSE_CANCEL);
+		else
+			gedit_dialog_add_button (GTK_DIALOG (msgbox),
+					_("_Don't close"), GTK_STOCK_CANCEL,
+                	             	GTK_RESPONSE_CANCEL);
 
 		gtk_dialog_add_button (GTK_DIALOG (msgbox),
-                             GTK_STOCK_CANCEL,
-                             GTK_RESPONSE_CANCEL);
-
-		gtk_dialog_add_button (GTK_DIALOG (msgbox),
-                             GTK_STOCK_SAVE,
-                             GTK_RESPONSE_YES);
+			       	GTK_STOCK_SAVE,
+				GTK_RESPONSE_YES);
 
 		gtk_dialog_set_default_response	(GTK_DIALOG (msgbox), GTK_RESPONSE_YES);
 
+		gtk_window_set_resizable (GTK_WINDOW (msgbox), FALSE);
+
 		ret = gtk_dialog_run (GTK_DIALOG (msgbox));
 		
 		gtk_widget_destroy (msgbox);
@@ -521,6 +636,11 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 		gedit_debug (DEBUG_MDI, "CLOSE: %s", close ? "TRUE" : "FALSE");
 	}
 	
+	/* FIXME: there is a bug if you "close all" >1 docs, don't save the document
+	 * and then don't close the last one.
+	 */
+	/* Disable to avoid the bug */
+	/*
 	if (close)
 	{
 		gtk_signal_disconnect_by_func (GTK_OBJECT (child), 
@@ -530,6 +650,7 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 				       GTK_SIGNAL_FUNC (gedit_mdi_child_undo_redo_state_changed_handler),
 				       NULL);
 	}
+	*/
 	
 	return close;
 }
@@ -566,17 +687,17 @@ gedit_mdi_set_active_window_title (BonoboMDI *mdi)
 
 	if (gedit_document_get_modified (doc))
 	{
-		title = g_strdup_printf ("gedit - [%s] %s", docname, _("(modified)"));
+		title = g_strdup_printf ("%s %s - gedit", docname, _("(modified)"));
 	} 
 	else 
 	{
 		if (gedit_document_is_readonly (doc)) 
 		{
-			title = g_strdup_printf ("gedit - [%s] %s", docname, _("(readonly)"));
+			title = g_strdup_printf ("%s %s - gedit", docname, _("(readonly)"));
 		} 
 		else 
 		{
-			title = g_strdup_printf ("gedit - [%s]", docname);
+			title = g_strdup_printf ("%s - gedit", docname);
 		}
 
 	}
@@ -596,11 +717,32 @@ void gedit_mdi_child_changed_handler (BonoboMDI *mdi, BonoboMDIChild *old_child)
 static 
 void gedit_mdi_view_changed_handler (BonoboMDI *mdi, GtkWidget *old_view)
 {
+	BonoboWindow *win;
+	GtkWidget* status;
+	GeditView *active_view;
+	
 	gedit_debug (DEBUG_MDI, "");
 
 	gedit_mdi_set_active_window_verbs_sensitivity (mdi);
 
-	gtk_widget_grab_focus (bonobo_mdi_get_active_view (mdi));
+	active_view = GEDIT_VIEW (bonobo_mdi_get_active_view (mdi));
+	
+	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	
+	win = bonobo_mdi_get_active_window (mdi);
+	g_return_if_fail (win != NULL);
+
+	if (old_view != NULL)
+	{
+		gedit_view_set_cursor_position_statusbar (GEDIT_VIEW (old_view), NULL);
+		gedit_view_set_overwrite_mode_statusbar (GEDIT_VIEW (old_view), NULL);
+	}
+
+	status = g_object_get_data (G_OBJECT (win), "CursorPosition");	
+	gedit_view_set_cursor_position_statusbar (active_view, status);
+
+	status = g_object_get_data (G_OBJECT (win), "OverwriteMode");	
+	gedit_view_set_overwrite_mode_statusbar (active_view, status);
 }
 
 void 
@@ -616,7 +758,9 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 	gedit_debug (DEBUG_MDI, "");
 	
 	active_window = bonobo_mdi_get_active_window (mdi);
-	g_return_if_fail (active_window != NULL);
+
+	if (active_window == NULL)
+		return;
 	
 	ui_engine = bonobo_window_get_ui_engine (active_window);
 	
@@ -707,6 +851,9 @@ gedit_mdi_update_ui_according_to_preferences (GeditMDI *mdi)
 	GList *children;
 	GdkColor background;
 	GdkColor text;
+	GdkColor selection;
+	GdkColor sel_text;
+	const gchar* font;
 
 	gedit_debug (DEBUG_MDI, "");
 
@@ -733,13 +880,44 @@ gedit_mdi_update_ui_according_to_preferences (GeditMDI *mdi)
 
 	children = bonobo_mdi_get_children (BONOBO_MDI (mdi));
 
-	background.red = settings->bg [0];
-	background.green = settings->bg [1];
-	background.blue = settings->bg [2];
+	if (gedit_settings->use_default_colors)
+	{
+		GtkStyle *style;
+	       
+		style = gtk_style_new ();
 
-	text.red = settings->fg [0];
-	text.green = settings->fg [1];
-	text.blue = settings->fg [2];
+		background = style->base [GTK_STATE_NORMAL];
+		text = style->text [GTK_STATE_NORMAL];
+		sel_text = style->text [GTK_STATE_SELECTED];
+		selection = style->base [GTK_STATE_SELECTED];
+
+		gtk_style_unref (style);
+	}
+	else
+	{
+		background = gedit_settings->background_color;
+		text = gedit_settings->text_color;
+		selection = gedit_settings->selection_color;
+		sel_text = gedit_settings->selected_text_color;
+	}
+
+	if (gedit_settings->use_default_font)
+	{
+		GtkStyle *style;
+		
+		style = gtk_style_new ();
+
+		font = pango_font_description_to_string (style->font_desc);
+		
+		if (font == NULL)
+			/* Fallback */
+			font = gedit_settings->editor_font;
+
+		gtk_style_unref (style);
+
+	}
+	else
+		font = gedit_settings->editor_font;
 
 	while (children != NULL)
 	{
@@ -749,9 +927,10 @@ gedit_mdi_update_ui_according_to_preferences (GeditMDI *mdi)
 		{
 			GeditView *v =	GEDIT_VIEW (views->data);
 			
-			gedit_view_set_colors (v, &background, &text);
-			gedit_view_set_font (v, settings->font);
-			gedit_view_set_wrap_mode (v, settings->wrap_mode);
+			gedit_view_set_colors (v, &background, &text, &selection, &sel_text);
+			gedit_view_set_font (v, font);
+			gedit_view_set_wrap_mode (v, gedit_settings->wrap_mode);
+			gedit_view_show_line_numbers (v, gedit_settings->show_line_numbers);
 			views = views->next;
 		}
 		
@@ -759,14 +938,6 @@ gedit_mdi_update_ui_according_to_preferences (GeditMDI *mdi)
 	}
 
 /*
-	bonobo_mdi_set_mode (BONOBO_MDI (mdi), settings->mdi_mode);
+	bonobo_mdi_set_mode (BONOBO_MDI (mdi), gedit_settings->mdi_mode);
 */
 }
-
-
-
-
-		
-
-
-		
